@@ -1,46 +1,68 @@
-"""The xonsh built-ins. Note that this module is named 'built_ins' so as
-not to be confused with the special Python builtins module.
+# -*- coding: utf-8 -*-
+"""The xonsh built-ins.
+
+Note that this module is named 'built_ins' so as not to be confused with the
+special Python builtins module.
 """
+import atexit
+import builtins
+from collections import Sequence
+from contextlib import contextmanager
+import inspect
+from glob import iglob
 import os
 import re
-import sys
-import time
 import shlex
-import atexit
 import signal
-import inspect
-import builtins
-import subprocess
-from io import TextIOWrapper, StringIO
-from glob import glob, iglob
-from subprocess import Popen, PIPE, STDOUT
-from contextlib import contextmanager
-from collections import Sequence, MutableMapping, Iterable, namedtuple, \
-    MutableSequence, MutableSet
+from subprocess import Popen, PIPE, STDOUT, CalledProcessError
+import sys
+import tempfile
+import time
 
-from xonsh.tools import suggest_commands, XonshError, ON_POSIX, ON_WINDOWS, \
-    string_types
-from xonsh.inspectors import Inspector
-from xonsh.environ import Env, default_env
-from xonsh.aliases import DEFAULT_ALIASES, bash_aliases
-from xonsh.jobs import add_job, wait_for_active_job
-from xonsh.proc import ProcProxy, SimpleProcProxy, TeePTYProc
+from xonsh.aliases import Aliases, make_default_aliases
+from xonsh.environ import Env, default_env, locate_binary
+from xonsh.foreign_shells import load_foreign_aliases
 from xonsh.history import History
+from xonsh.inspectors import Inspector
+from xonsh.jobs import add_job, wait_for_active_job
+from xonsh.platform import ON_POSIX, ON_WINDOWS
+from xonsh.proc import (ProcProxy, SimpleProcProxy, ForegroundProcProxy,
+                        SimpleForegroundProcProxy, TeePTYProc,
+                        CompletedCommand, HiddenCompletedCommand)
+from xonsh.tools import (
+    suggest_commands, XonshError, expandvars, CommandsCache, globpath,
+    iglobpath
+)
+
 
 ENV = None
 BUILTINS_LOADED = False
 INSPECTOR = Inspector()
-AT_EXIT_SIGNALS = (signal.SIGABRT, signal.SIGFPE, signal.SIGILL, signal.SIGSEGV, 
+AT_EXIT_SIGNALS = (signal.SIGABRT, signal.SIGFPE, signal.SIGILL, signal.SIGSEGV,
                    signal.SIGTERM)
+
+SIGNAL_MESSAGES = {
+    signal.SIGABRT: 'Aborted',
+    signal.SIGFPE: 'Floating point exception',
+    signal.SIGILL: 'Illegal instructions',
+    signal.SIGTERM: 'Terminated',
+    signal.SIGSEGV: 'Segmentation fault'
+}
+
 if ON_POSIX:
     AT_EXIT_SIGNALS += (signal.SIGTSTP, signal.SIGQUIT, signal.SIGHUP)
-
+    SIGNAL_MESSAGES.update({
+        signal.SIGQUIT: 'Quit',
+        signal.SIGHUP: 'Hangup',
+        signal.SIGKILL: 'Killed'
+    })
 
 def resetting_signal_handle(sig, f):
-    """Sets a new signal handle that will automaticallly restore the old value 
+    """Sets a new signal handle that will automatically restore the old value
     once the new handle is finished.
     """
     oldh = signal.getsignal(sig)
+
     def newh(s=None, frame=None):
         f(s, frame)
         signal.signal(sig, oldh)
@@ -49,105 +71,6 @@ def resetting_signal_handle(sig, f):
     signal.signal(sig, newh)
 
 
-class Aliases(MutableMapping):
-    """Represents a location to hold and look up aliases."""
-
-    def __init__(self, *args, **kwargs):
-        self._raw = {}
-        self.update(*args, **kwargs)
-
-    def get(self, key, default=None):
-        """Returns the (possibly modified) value. If the key is not present,
-        then `default` is returned.
-        If the value is callable, it is returned without modification. If it
-        is an iterable of strings it will be evaluated recursively to expand
-        other aliases, resulting in a new list or a "partially applied"
-        callable.
-        """
-        val = self._raw.get(key)
-        if val is None:
-            return default
-        elif isinstance(val, Iterable) or callable(val):
-            return self.eval_alias(val, seen_tokens={key})
-        else:
-            msg = 'alias of {!r} has an inappropriate type: {!r}'
-            raise TypeError(msg.format(key, val))
-
-    def eval_alias(self, value, seen_tokens, acc_args=[]):
-        """
-        "Evaluates" the alias `value`, by recursively looking up the leftmost
-        token and "expanding" if it's also an alias.
-
-        A value like ["cmd", "arg"] might transform like this:
-        > ["cmd", "arg"] -> ["ls", "-al", "arg"] -> callable()
-        where `cmd=ls -al` and `ls` is an alias with its value being a
-        callable.  The resulting callable will be "partially applied" with
-        ["-al", "arg"].
-        """
-        # Beware of mutability: default values for keyword args are evaluated
-        # only once.
-        if callable(value):
-            if acc_args:  # Partial application
-                return lambda args, stdin=None: value(acc_args + args,
-                                                      stdin=stdin)
-            else:
-                return value
-        else:
-            token, *rest = value
-            if token in seen_tokens or token not in self._raw:
-                # ^ Making sure things like `egrep=egrep --color=auto` works,
-                # and that `l` evals to `ls --color=auto -CF` if `l=ls -CF`
-                # and `ls=ls --color=auto`
-                return value + acc_args
-            else:
-                return self.eval_alias(self._raw[token],
-                                       seen_tokens | {token},
-                                       rest + acc_args)
-
-    #
-    # Mutable mapping interface
-    #
-
-    def __getitem__(self, key):
-        return self._raw[key]
-
-    def __setitem__(self, key, val):
-        if isinstance(val, string_types):
-            self._raw[key] = shlex.split(val)
-        else:
-            self._raw[key] = val
-
-    def __delitem__(self, key):
-        del self._raw[key]
-
-    def update(self, *args, **kwargs):
-        for key, val in dict(*args, **kwargs).items():
-            self[key] = val
-
-    def __iter__(self):
-        yield from self._raw
-
-    def __len__(self):
-        return len(self._raw)
-
-    def __str__(self):
-        return str(self._raw)
-
-    def __repr__(self):
-        return '{0}.{1}({2})'.format(self.__class__.__module__,
-                                     self.__class__.__name__, self._raw)
-
-    def _repr_pretty_(self, p, cycle):
-        name = '{0}.{1}'.format(self.__class__.__module__,
-                                self.__class__.__name__)
-        with p.group(0, name + '(', ')'):
-            if cycle:
-                p.text('...')
-            elif len(self):
-                p.break_()
-                p.pretty(dict(self))
-
-
 def helper(x, name=''):
     """Prints help about, and then returns that variable."""
     INSPECTOR.pinfo(x, oname=name, detail_level=0)
@@ -163,39 +86,9 @@ def superhelper(x, name=''):
 def expand_path(s):
     """Takes a string path and expands ~ to home and environment vars."""
     global ENV
-    if ENV is not None:
-        ENV.replace_env()
-    return os.path.expanduser(os.path.expandvars(s))
-
-
-def expand_case_matching(s):
-    """Expands a string to a case insenstive globable string."""
-    t = []
-    openers = {'[', '{'}
-    closers = {']', '}'}
-    nesting = 0
-    for c in s:
-        if c in openers:
-            nesting += 1
-        elif c in closers:
-            nesting -= 1
-        elif nesting > 0:
-            pass
-        elif c.isalpha():
-            folded = c.casefold()
-            if len(folded) == 1:
-                c = '[{0}{1}]'.format(c.upper(), c.lower())
-            else:
-                newc = ['[{0}{1}]?'.format(f.upper(), f.lower())
-                        for f in folded[:-1]]
-                newc = ''.join(newc)
-                newc += '[{0}{1}{2}]'.format(folded[-1].upper(),
-                                             folded[-1].lower(),
-                                             c)
-                c = newc
-        t.append(c)
-    t = ''.join(t)
-    return t
+    if ENV.get('EXPAND_ENV_VARS'):
+        s = expandvars(s)
+    return os.path.expanduser(s)
 
 
 def reglob(path, parts=None, i=None):
@@ -226,65 +119,29 @@ def reglob(path, parts=None, i=None):
     if i1 == len(parts):
         for f in files:
             p = os.path.join(base, f)
-            if regex.match(p) is not None:
+            if regex.fullmatch(p) is not None:
                 paths.append(p)
     else:
         for f in files:
             p = os.path.join(base, f)
-            if regex.match(p) is None or not os.path.isdir(p):
+            if regex.fullmatch(p) is None or not os.path.isdir(p):
                 continue
             paths += reglob(p, parts=parts, i=i1)
     return paths
 
 
-def regexpath(s):
+def regexpath(s, pymode=False):
     """Takes a regular expression string and returns a list of file
     paths that match the regex.
     """
     s = expand_path(s)
-    return reglob(s)
-
-
-def globpath(s, ignore_case=False):
-    """Simple wrapper around glob that also expands home and env vars."""
-    s = expand_path(s)
-    if ignore_case:
-        s = expand_case_matching(s)
-    o = glob(s)
-    return o if len(o) != 0 else [s]
-
-
-def iglobpath(s, ignore_case=False):
-    """Simple wrapper around iglob that also expands home and env vars."""
-    s = expand_path(s)
-    if ignore_case:
-        s = expand_case_matching(s)
-    return iglob(s)
-
+    o = reglob(s)
+    no_match = [] if pymode else [s]
+    return o if len(o) != 0 else no_match
 
 RE_SHEBANG = re.compile(r'#![ \t]*(.+?)$')
 
 
-def _get_runnable_name(fname):
-    if os.path.isfile(fname) and fname != os.path.basename(fname):
-        return fname
-    for d in builtins.__xonsh_env__['PATH']:
-        if os.path.isdir(d):
-            files = os.listdir(d)
-
-            if ON_WINDOWS:
-                PATHEXT = builtins.__xonsh_env__.get('PATHEXT', [])
-                for dirfile in files:
-                    froot, ext = os.path.splitext(dirfile)
-                    if fname == froot and ext.upper() in PATHEXT:
-                        return os.path.join(d, dirfile)
-
-            if fname in files:
-                return os.path.join(d, fname)
-
-    return None
-
-
 def _is_binary(fname, limit=80):
     with open(fname, 'rb') as f:
         for i in range(limit):
@@ -321,10 +178,10 @@ def get_script_subproc_command(fname, args):
         raise PermissionError
 
     if ON_POSIX and not os.access(fname, os.R_OK):
-        # on some systems, some importnat programs (e.g. sudo) will have execute 
-        # permissions but not read/write permisions. This enables things with the SUID 
-        # set to be run. Needs to come before _is_binary() is called, because that 
-        # function tries to read the file.
+        # on some systems, some importnat programs (e.g. sudo) will have
+        # execute permissions but not read/write permisions. This enables
+        # things with the SUID set to be run. Needs to come before _is_binary()
+        # is called, because that function tries to read the file.
         return [fname] + args
     elif _is_binary(fname):
         # if the file is a binary, we should call it directly
@@ -334,7 +191,7 @@ def get_script_subproc_command(fname, args):
         # Windows can execute various filetypes directly
         # as given in PATHEXT
         _, ext = os.path.splitext(fname)
-        if ext.upper() in builtins.__xonsh_env__.get('PATHEXT', []):
+        if ext.upper() in builtins.__xonsh_env__.get('PATHEXT'):
             return [fname] + args
 
     # find interpreter
@@ -389,8 +246,12 @@ def _open(fname, mode):
         return fname
     try:
         return open(fname, mode)
-    except Exception:
+    except PermissionError:
+        raise XonshError('xonsh: {0}: permission denied'.format(fname))
+    except FileNotFoundError:
         raise XonshError('xonsh: {0}: no such file or directory'.format(fname))
+    except Exception:
+        raise XonshError('xonsh: {0}: unable to open file'.format(fname))
 
 
 def _redirect_io(streams, r, loc=None):
@@ -398,7 +259,7 @@ def _redirect_io(streams, r, loc=None):
     if r.replace('&', '') in _E2O_MAP:
         if 'stderr' in streams:
             raise XonshError('Multiple redirects for stderr')
-        streams['stderr'] = STDOUT
+        streams['stderr'] = ('<stdout>', 'a', STDOUT)
         return
 
     orig, mode, dest = _REDIR_REGEX.match(r).groups()
@@ -425,7 +286,7 @@ def _redirect_io(streams, r, loc=None):
         elif 'stdin' in streams:
             raise XonshError('Multiple inputs for stdin')
         else:
-            streams['stdin'] = _open(loc, mode)
+            streams['stdin'] = (loc, 'r', _open(loc, mode))
     elif mode in _WRITE_MODES:
         if orig in _REDIR_ALL:
             if 'stderr' in streams:
@@ -455,13 +316,13 @@ def _redirect_io(streams, r, loc=None):
 
         f = _open(loc, mode)
         for t in targets:
-            streams[t] = f
+            streams[t] = (loc, mode, f)
 
     else:
         raise XonshError('Unrecognized redirection command: {}'.format(r))
 
 
-def run_subproc(cmds, captured=True):
+def run_subproc(cmds, captured=False):
     """Runs a subprocess, in its many forms. This takes a list of 'commands,'
     which may be a list of command line arguments or a string, representing
     a special connecting character.  For example::
@@ -476,20 +337,21 @@ def run_subproc(cmds, captured=True):
     """
     global ENV
     background = False
+    procinfo = {}
     if cmds[-1] == '&':
         background = True
         cmds = cmds[:-1]
     write_target = None
     last_cmd = len(cmds) - 1
-    prev = None
     procs = []
     prev_proc = None
+    _capture_streams = captured in {'stdout', 'object'}
     for ix, cmd in enumerate(cmds):
+        starttime = time.time()
+        procinfo['args'] = list(cmd)
         stdin = None
-        stdout = None
         stderr = None
-        if isinstance(cmd, string_types):
-            prev = cmd
+        if isinstance(cmd, str):
             continue
         streams = {}
         while True:
@@ -508,29 +370,53 @@ def run_subproc(cmds, captured=True):
         if 'stdin' in streams:
             if prev_proc is not None:
                 raise XonshError('Multiple inputs for stdin')
-            stdin = streams['stdin']
+            stdin = streams['stdin'][-1]
+            procinfo['stdin_redirect'] = streams['stdin'][:-1]
         elif prev_proc is not None:
             stdin = prev_proc.stdout
         # set standard output
+        _stdout_name = None
+        _stderr_name = None
         if 'stdout' in streams:
             if ix != last_cmd:
                 raise XonshError('Multiple redirects for stdout')
-            stdout = streams['stdout']
-        elif captured or ix != last_cmd:
+            stdout = streams['stdout'][-1]
+            procinfo['stdout_redirect'] = streams['stdout'][:-1]
+        elif ix != last_cmd:
             stdout = PIPE
+        elif _capture_streams:
+            _nstdout = stdout = tempfile.NamedTemporaryFile(delete=False)
+            _stdout_name = stdout.name
+        elif builtins.__xonsh_stdout_uncaptured__ is not None:
+            stdout = builtins.__xonsh_stdout_uncaptured__
         else:
             stdout = None
         # set standard error
         if 'stderr' in streams:
-            stderr = streams['stderr']
-        uninew = ix == last_cmd
+            stderr = streams['stderr'][-1]
+            procinfo['stderr_redirect'] = streams['stderr'][:-1]
+        elif captured == 'object' and ix == last_cmd:
+            _nstderr = stderr = tempfile.NamedTemporaryFile(delete=False)
+            _stderr_name = stderr.name
+        elif builtins.__xonsh_stderr_uncaptured__ is not None:
+            stderr = builtins.__xonsh_stderr_uncaptured__
+        uninew = (ix == last_cmd) and (not _capture_streams)
         alias = builtins.aliases.get(cmd[0], None)
+        procinfo['alias'] = alias
+        if (alias is None and
+                builtins.__xonsh_env__.get('AUTO_CD') and
+                len(cmd) == 1 and
+                os.path.isdir(cmd[0]) and
+                locate_binary(cmd[0]) is None):
+            cmd.insert(0, 'cd')
+            alias = builtins.aliases.get('cd', None)
+
         if callable(alias):
             aliased_cmd = alias
         else:
             if alias is not None:
                 cmd = alias + cmd[1:]
-            n = _get_runnable_name(cmd[0])
+            n = locate_binary(cmd[0])
             if n is None:
                 aliased_cmd = cmd
             else:
@@ -539,13 +425,28 @@ def run_subproc(cmds, captured=True):
                 except PermissionError:
                     e = 'xonsh: subprocess mode: permission denied: {0}'
                     raise XonshError(e.format(cmd[0]))
+        _stdin_file = None
+        if (stdin is not None and
+                ENV.get('XONSH_STORE_STDIN') and
+                captured == 'object' and
+                'cat' in __xonsh_commands_cache__ and
+                'tee' in __xonsh_commands_cache__):
+            _stdin_file = tempfile.NamedTemporaryFile()
+            cproc = Popen(['cat'],
+                          stdin=stdin,
+                          stdout=PIPE)
+            tproc = Popen(['tee', _stdin_file.name],
+                          stdin=cproc.stdout,
+                          stdout=PIPE)
+            stdin = tproc.stdout
         if callable(aliased_cmd):
             prev_is_proxy = True
+            bgable = getattr(aliased_cmd, '__xonsh_backgroundable__', True)
             numargs = len(inspect.signature(aliased_cmd).parameters)
             if numargs == 2:
-                cls = SimpleProcProxy
+                cls = SimpleProcProxy if bgable else SimpleForegroundProcProxy
             elif numargs == 4:
-                cls = ProcProxy
+                cls = ProcProxy if bgable else ForegroundProcProxy
             else:
                 e = 'Expected callable with 2 or 4 arguments, not {}'
                 raise XonshError(e.format(numargs))
@@ -554,8 +455,9 @@ def run_subproc(cmds, captured=True):
                        universal_newlines=uninew)
         else:
             prev_is_proxy = False
-            usetee = (stdout is None) and (not background) and \
-                     ENV.get('XONSH_STORE_STDOUT', False)
+            usetee = ((stdout is None) and
+                      (not background) and
+                      ENV.get('XONSH_STORE_STDOUT', False))
             cls = TeePTYProc if usetee else Popen
             subproc_kwargs = {}
             if ON_POSIX and cls is Popen:
@@ -579,7 +481,6 @@ def run_subproc(cmds, captured=True):
                     e += '\n' + suggest_commands(cmd, ENV, builtins.aliases)
                 raise XonshError(e)
         procs.append(proc)
-        prev = None
         prev_proc = proc
     for proc in procs[:-1]:
         try:
@@ -593,6 +494,14 @@ def run_subproc(cmds, captured=True):
             'obj': prev_proc,
             'bg': background
         })
+    if (ENV.get('XONSH_INTERACTIVE') and
+            not ENV.get('XONSH_STORE_STDOUT') and
+            not _capture_streams):
+        # set title here to get current command running
+        try:
+            builtins.__xonsh_shell__.settitle()
+        except AttributeError:
+            pass
     if background:
         return
     if prev_is_proxy:
@@ -600,22 +509,102 @@ def run_subproc(cmds, captured=True):
     wait_for_active_job()
     hist = builtins.__xonsh_history__
     hist.last_cmd_rtn = prev_proc.returncode
+    # get output
+    output = b''
     if write_target is None:
-        # get output
-        output = ''
-        if prev_proc.stdout not in (None, sys.stdout):
+        if _stdout_name is not None:
+            with open(_stdout_name, 'rb') as stdoutfile:
+                output = stdoutfile.read()
+            try:
+                _nstdout.close()
+            except:
+                pass
+            os.unlink(_stdout_name)
+        elif prev_proc.stdout not in (None, sys.stdout):
             output = prev_proc.stdout.read()
-        if captured:
-            return output
+        if _capture_streams:
+            # to get proper encoding from Popen, we have to
+            # use a byte stream and then implement universal_newlines here
+            output = output.decode(encoding=ENV.get('XONSH_ENCODING'),
+                                   errors=ENV.get('XONSH_ENCODING_ERRORS'))
+            output = output.replace('\r\n', '\n')
         else:
             hist.last_cmd_out = output
+        if captured == 'object': # get stderr as well
+            named = _stderr_name is not None
+            unnamed = prev_proc.stderr not in {None, sys.stderr}
+            if named:
+                with open(_stderr_name, 'rb') as stderrfile:
+                    errout = stderrfile.read()
+                try:
+                    _nstderr.close()
+                except:
+                    pass
+                os.unlink(_stderr_name)
+            elif unnamed:
+                errout = prev_proc.stderr.read()
+            if named or unnamed:
+                errout = errout.decode(encoding=ENV.get('XONSH_ENCODING'),
+                                       errors=ENV.get('XONSH_ENCODING_ERRORS'))
+                errout = errout.replace('\r\n', '\n')
+                procinfo['stderr'] = errout
+
+    if getattr(prev_proc, 'signal', None):
+        sig, core = prev_proc.signal
+        sig_str = SIGNAL_MESSAGES.get(sig)
+        if sig_str:
+            if core:
+                sig_str += ' (core dumped)'
+            print(sig_str, file=sys.stderr)
+    if (not prev_is_proxy and
+            hist.last_cmd_rtn is not None and
+            hist.last_cmd_rtn > 0 and
+            ENV.get('RAISE_SUBPROC_ERROR')):
+        raise CalledProcessError(hist.last_cmd_rtn, aliased_cmd, output=output)
+    if captured == 'stdout':
+        return output
+    elif captured is not False:
+        procinfo['pid'] = prev_proc.pid
+        procinfo['returncode'] = prev_proc.returncode
+        procinfo['timestamp'] = (starttime, time.time())
+        if captured == 'object':
+            procinfo['stdout'] = output
+            if _stdin_file is not None:
+                _stdin_file.seek(0)
+                procinfo['stdin'] = _stdin_file.read().decode()
+                _stdin_file.close()
+            return CompletedCommand(**procinfo)
+        else:
+            return HiddenCompletedCommand(**procinfo)
 
 
-def subproc_captured(*cmds):
+def subproc_captured_stdout(*cmds):
     """Runs a subprocess, capturing the output. Returns the stdout
     that was produced as a str.
     """
-    return run_subproc(cmds, captured=True)
+    return run_subproc(cmds, captured='stdout')
+
+
+def subproc_captured_inject(*cmds):
+    """Runs a subprocess, capturing the output. Returns a list of
+    whitespace-separated strings in the stdout that was produced."""
+    return [i.strip() for i in run_subproc(cmds, captured='stdout').split()]
+
+
+def subproc_captured_object(*cmds):
+    """
+    Runs a subprocess, capturing the output. Returns an instance of
+    ``CompletedCommand`` representing the completed command.
+    """
+    return run_subproc(cmds, captured='object')
+
+
+def subproc_captured_hiddenobject(*cmds):
+    """
+    Runs a subprocess, capturing the output. Returns an instance of
+    ``HiddenCompletedCommand`` representing the completed command.
+    """
+    return run_subproc(cmds, captured='hiddenobject')
 
 
 def subproc_uncaptured(*cmds):
@@ -627,57 +616,69 @@ def subproc_uncaptured(*cmds):
 
 def ensure_list_of_strs(x):
     """Ensures that x is a list of strings."""
-    if isinstance(x, string_types):
+    if isinstance(x, str):
         rtn = [x]
     elif isinstance(x, Sequence):
-        rtn = [i if isinstance(i, string_types) else str(i) for i in x]
+        rtn = [i if isinstance(i, str) else str(i) for i in x]
     else:
         rtn = [str(x)]
     return rtn
 
 
-def load_builtins(execer=None):
+def load_builtins(execer=None, config=None, login=False, ctx=None):
     """Loads the xonsh builtins into the Python builtins. Sets the
     BUILTINS_LOADED variable to True.
     """
     global BUILTINS_LOADED, ENV
     # private built-ins
-    builtins.__xonsh_env__ = ENV = Env(default_env())
-    builtins.__xonsh_ctx__ = {}
+    builtins.__xonsh_config__ = {}
+    builtins.__xonsh_env__ = ENV = Env(default_env(config=config, login=login))
     builtins.__xonsh_help__ = helper
     builtins.__xonsh_superhelp__ = superhelper
     builtins.__xonsh_regexpath__ = regexpath
     builtins.__xonsh_glob__ = globpath
+    builtins.__xonsh_expand_path__ = expand_path
     builtins.__xonsh_exit__ = False
+    builtins.__xonsh_stdout_uncaptured__ = None
+    builtins.__xonsh_stderr_uncaptured__ = None
     if hasattr(builtins, 'exit'):
         builtins.__xonsh_pyexit__ = builtins.exit
         del builtins.exit
     if hasattr(builtins, 'quit'):
         builtins.__xonsh_pyquit__ = builtins.quit
         del builtins.quit
-    builtins.__xonsh_subproc_captured__ = subproc_captured
+    builtins.__xonsh_subproc_captured_stdout__ = subproc_captured_stdout
+    builtins.__xonsh_subproc_captured_inject__ = subproc_captured_inject
+    builtins.__xonsh_subproc_captured_object__ = subproc_captured_object
+    builtins.__xonsh_subproc_captured_hiddenobject__ = subproc_captured_hiddenobject
     builtins.__xonsh_subproc_uncaptured__ = subproc_uncaptured
     builtins.__xonsh_execer__ = execer
+    builtins.__xonsh_commands_cache__ = CommandsCache()
     builtins.__xonsh_all_jobs__ = {}
-    builtins.__xonsh_active_job__ = None
     builtins.__xonsh_ensure_list_of_strs__ = ensure_list_of_strs
     # public built-ins
     builtins.evalx = None if execer is None else execer.eval
     builtins.execx = None if execer is None else execer.exec
     builtins.compilex = None if execer is None else execer.compile
-    builtins.default_aliases = builtins.aliases = Aliases(DEFAULT_ALIASES)
-    builtins.aliases.update(bash_aliases())
+
+    # Need this inline/lazy import here since we use locate_binary that relies on __xonsh_env__ in default aliases
+    builtins.default_aliases = builtins.aliases = Aliases(make_default_aliases())
+    if login:
+        builtins.aliases.update(load_foreign_aliases(issue_warning=False))
     # history needs to be started after env and aliases
     # would be nice to actually include non-detyped versions.
-    builtins.__xonsh_history__ = History(env=ENV.detype(), #aliases=builtins.aliases, 
+    builtins.__xonsh_history__ = History(env=ENV.detype(),
                                          ts=[time.time(), None], locked=True)
-    lastflush = lambda s=None, f=None: builtins.__xonsh_history__.flush(at_exit=True)
-    atexit.register(lastflush)
+    atexit.register(_lastflush)
     for sig in AT_EXIT_SIGNALS:
-        resetting_signal_handle(sig, lastflush)
+        resetting_signal_handle(sig, _lastflush)
     BUILTINS_LOADED = True
 
 
+def _lastflush(s=None, f=None):
+    builtins.__xonsh_history__.flush(at_exit=True)
+
+
 def unload_builtins():
     """Removes the xonsh builtins from the Python builtins, if the
     BUILTINS_LOADED is True, sets BUILTINS_LOADED to False, and returns.
@@ -692,24 +693,31 @@ def unload_builtins():
         builtins.quit = builtins.__xonsh_pyquit__
     if not BUILTINS_LOADED:
         return
-    names = ['__xonsh_env__',
+    names = ['__xonsh_config__',
+             '__xonsh_env__',
              '__xonsh_ctx__',
              '__xonsh_help__',
              '__xonsh_superhelp__',
              '__xonsh_regexpath__',
              '__xonsh_glob__',
+             '__xonsh_expand_path__',
              '__xonsh_exit__',
+             '__xonsh_stdout_uncaptured__',
+             '__xonsh_stderr_uncaptured__',
              '__xonsh_pyexit__',
              '__xonsh_pyquit__',
-             '__xonsh_subproc_captured__',
+             '__xonsh_subproc_captured_stdout__',
+             '__xonsh_subproc_captured_inject__',
+             '__xonsh_subproc_captured_object__',
+             '__xonsh_subproc_captured_hiddenobject__',
              '__xonsh_subproc_uncaptured__',
              '__xonsh_execer__',
+             '__xonsh_commands_cache__',
              'evalx',
              'execx',
              'compilex',
              'default_aliases',
              '__xonsh_all_jobs__',
-             '__xonsh_active_job__',
              '__xonsh_ensure_list_of_strs__',
              '__xonsh_history__',
              ]
